/*********************************************************************
他のXBeeから送られてきたテキストデータをキャラクタＬＣＤに表示します。

このスケッチは子機用（M2M子機）です。センサーは、孫機となります。
親機用としても使えます。

本Arduino機に接続するXBeeモジュールは「ROUTER API」のファームウェアを
書き込んでください。
もちろん、「COORDINATOR API」を書き込むことで親機としても動作します。

	液晶はAdafruit LCD Sheildのライブラリを使用しています。
	お持ちでない場合は、#define ADAFRUITを無効にしてください。
	
本ソースリストおよびソフトウェアは、ライセンスフリーです。
利用、編集、再配布等が自由に行えますが、著作権表示の改変は禁止します。

                               Copyright (c) 2013- Wataru KUNINO
                               http://www.geocities.jp/bokunimowakaru/
*********************************************************************/
/*
機能説明

・テキストを受信すると改行までの文字をＬＣＤへ出力します。
・表示しきれない場合はスクロール表示します。
・エスケープコマンド(0x1B)を受信するとコマンドに応じてレベル表示などを
　行います。
*/
/*
画面設計

	起動時					メニュー表示			レベル表示時
	　0123456789012345		　0123456789012345		　0123456789012345
	┏━━━━━━━━┓	┏━━━━━━━━┓	┏━━━━━━━━┓
	┃M2M Router AI=00┃	┃LED_SW        00┃	┃----- ----- I=00┃
	┃text message....┃	┃Sent GPIO(11)= 1┃	┃26.5C 3000W 1000┃
	┗━━━━━━━━┛	┗━━━━━━━━┛	┗━━━━━━━━┛
	
*/
/*
Arduinoへの書き込み時の注意点

書き込み時にXBeeからの受信があると、書き込みに失敗する場合があります。
全ての子機をオフにする、もしくはArduinoに取り付けた親機を外してから
書き込んでください。
*/
/*********************************************************************
インポート処理						※ライブラリ XBeeに必要な宣言です
*********************************************************************/

#include <xbee.h>

/*********************************************************************
アプリケーション					※以下を編集してアプリを作成します
*********************************************************************/

#define ADAFRUIT	// 使用する液晶が Adafruit LCD Sheild の時に定義する

#ifndef ADAFRUIT
	// 通常のキャラクタLCDの場合
	#include <LiquidCrystal.h>
	LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
#else
	// Adafruit I2C接続LCDの場合(Adafruit_RGBLCDShieldライブラリが必要)
	#include <Wire.h>
	#include <Adafruit_MCP23017.h>
	#include <Adafruit_RGBLCDShield.h>
	Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();
#endif

//　↓お手持ちのXBeeモジュール孫機のIEEEアドレスに変更する
byte dev_mago[]   = {0x00,0x13,0xA2,0x00,0x40,0x30,0xC1,0x6F};	// GPIOデバイス
byte dev_sens[]   = {0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF};	// センサデバイス
byte dev_uart[]   = {0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF};	// UARTデバイス

byte font_lv[4][8]={
	{
		0b00000,
		0b10000,
		0b00000,
		0b10000,
		0b00000,
		0b10000,
		0b00000,
		0b10101
	},{
		0b11000,
		0b11000,
		0b11000,
		0b11000,
		0b11000,
		0b11000,
		0b11000,
		0b10101
	},{
		0b11011,
		0b11011,
		0b11011,
		0b11011,
		0b11011,
		0b11011,
		0b11011,
		0b10101
	},{
		0b00000,
		0b10000,
		0b00000,
		0b10000,
		0b00000,
		0b10000,
		0b00000,
		0b10000
	}
};
/*
	　０１２３４　０１２３４　０１２３４　０１２３４　０１２３４
	０●　　　　　●　　　　　●　　　　　●　　　　　●　　　　
	１
	２●　　　　　●　　　　　●　　　　　●　　　　　●　　　　
	３
	４●　　　　　●　　　　　●　　　　　●　　　　　●　　　　
	５
	６●　●　●　●　●　●　●　●　●　●　●　●　●　●　●
	
	　０１２３４　０１２３４　０１２３４　０１２３４　０１２３４
	０●●　　　　●　　　　　●　　　　　●　　　　　●　　　　
	１●●
	２●●　　　　●　　　　　●　　　　　●　　　　　●　　　　
	３●●
	４●●　　　　●　　　　　●　　　　　●　　　　　●　　　　
	５●●
	６●　●　●　●　●　●　●　●　●　●　●　●　●　●　●
	
	　０１２３４　０１２３４　０１２３４　０１２３４　０１２３４
	０●●　●●　●　　　　　●　　　　　●　　　　　●　　　　
	１●●　●●
	２●●　●●　●　　　　　●　　　　　●　　　　　●　　　　
	３●●　●●
	４●●　●●　●　　　　　●　　　　　●　　　　　●　　　　
	５●●　●●
	６●　●　●　●　●　●　●　●　●　●　●　●　●　●　●
	

	　０１２３４　０１２３４　０１２３４　０１２３４　０１２３４
	０●●　●●　●●　●●　●●　●●　●●　●●　●●　●●
	１●●　●●　●●　●●　●●　●●　●●　●●　●●　●●
	２●●　●●　●●　●●　●●　●●　●●　●●　●●　●●
	３●●　●●　●●　●●　●●　●●　●●　●●　●●　●●
	４●●　●●　●●　●●　●●　●●　●●　●●　●●　●●
	５●●　●●　●●　●●　●●　●●　●●　●●　●●　●●
	６●　●　●　●　●　●　●　●　●　●　●　●　●　●　●

*/

void lcd_print_level(const byte level_1, const byte level_2){
	byte i;
	
	if( level_1 == 0 && level_2 == 0){
		lcd.setCursor( 0 , 0 );
		lcd.print( "M2M Router  I=" );
	}else{
		lcd.setCursor( 0 , 0 );
		for(i=0 ; i < 5 ; i++){
			if( level_1 <= 2 * i )			lcd.write( (byte)0x00 );
			else if( level_1 - 2 * i == 1 )	lcd.write( (byte)0x01 );
			else							lcd.write( (byte)0x02 );
		}
		lcd.write(3);
		for(i=0 ; i < 5 ; i++){
			lcd.setCursor( i+6 , 0 );
			if( level_2 <= 2 * i )			lcd.write( (byte)0x00 );
			else if( level_2 - 2 * i == 1 )	lcd.write( (byte)0x01 );
			else							lcd.write( (byte)0x02 );
		}
		lcd.write(3);
		lcd.print("I=");
	}
}

void setup(){
	lcd.begin(16, 2);
	lcd.clear(); lcd.print("Sample 11 LCD");
	xbee_init( 0x00 );				// XBee用COMポートの初期化
	lcd.setCursor(0,1);
	lcd.print("by Wataru Kunino");
	xbee_atnj( 0 );					// ペアリング禁止
	xbee_gpio_init( dev_mago );		// XBee孫機のGPIO設定
	for( byte i=0 ; i < 4 ; i++ ){
		lcd.createChar( i , &(font_lv[i][0]) );
	}
}

void loop(){
	byte i,counter;
	byte pan_id[8];
	unsigned short pan_id_s;
	byte data;
	byte buttons;
	unsigned int value_i;
	byte menu_n = 0;
	char menu[5][7] = { "LED_SW", "TxUART", "force",  "PAN_ID",  "ATNR0" };
	byte led = 0x00;
	
	// テキスト表示   0123456789012345
	char strings[] = "by Wataru Kunino";
	byte str_len   = 0;
	byte lcd_clear = 0x01;
	byte level_1   = 0;
	byte level_2   = 0;
	byte alert_value = 0;

	XBEE_RESULT xbee_result;			// 受信データ用の構造体

	// メイン処理
	while(1){							// 永久に受信する
		buttons = lcd.readButtons();

		if( counter >= 200 ){
			if( (lcd_clear&0x02) == 0x02 ){	// UI表示中の時に画面を消去
				lcd.clear();
				lcd_print_level( level_1, level_2);
				lcd_clear &= 0xFD;			// フラグのクリア
			}
			lcd.home();
			lcd.setCursor(14,0);
			lcd.print( lcd.print( xbee_atai() , HEX ) );
			lcd.setCursor(0,1);
			lcd.print( strings );
			counter = 0;
		}
		counter++;
		if( buttons != 0 ){
			lcd_clear |= 0x02;	// 次回のUI画面消去用のフラグのセット
			counter = 0;
			switch( buttons ){
				case BUTTON_UP:
				//	do{
						xbee_at("ATCB01");			// 親機へのジョイン開始
						delay(3000);
						pan_id_s = xbee_atop( pan_id );			// PAN_IDの取得
						lcd.clear();
						lcd.print("PAN ID=");
						lcd.print( pan_id_s, HEX );
						lcd.setCursor(0,1);
						for(i=0;i<8;i++){
							lcd.print( pan_id[i]>>4 , HEX );
							lcd.print( pan_id[i]%16 , HEX );
						}
				//	}while( xbee_atai() > 0x20 );
					break;
				case BUTTON_DOWN:
					lcd.clear(); lcd.print("ATNJ=16");
					xbee_at("ATNJ10");				// 孫機のジョイン許可
					break;
				case BUTTON_LEFT:
					if( menu_n > 0 ) menu_n--; else menu_n = 4;
					lcd.clear(); lcd.print( menu[menu_n] );
					break;
				case BUTTON_RIGHT:
					if( menu_n < 4 ) menu_n++; else menu_n = 0;
					lcd.clear(); lcd.print( menu[menu_n] );
					break;
				case BUTTON_SELECT:
					lcd.clear(); lcd.print( menu[menu_n] );
					switch( menu_n ){
						case 0:		// "LED_SW"
							if( led == 0x00 ) led = 0x01; else led = 0x00;
							xbee_gpo( dev_mago , 11 , led );
							lcd.setCursor(0,1);
							lcd.print("Sent GPIO(11)= ");
							lcd.print( led , BIN );
							break;
						case 1:		// "TxUART"
							xbee_address( dev_uart );
							xbee_putstr("Hello, M2M!");
							lcd.setCursor(0,1);
							lcd.print("Sent Hello, M2M!");
							break;
						case 2:		// "force"
							xbee_force( dev_sens );
							lcd.setCursor(0,1);
							lcd.print("Sent force IS");
							break;
						case 3:		// "PAN_ID"
							pan_id_s = xbee_atop( pan_id );			// PAN_IDの取得
							lcd.setCursor(7,0);
							lcd.print( pan_id_s, HEX );
							lcd.setCursor(0,1);
							for(i=0;i<8;i++){
								lcd.print( pan_id[i]>>4 , HEX );
								lcd.print( pan_id[i]%16 , HEX );
							}
							break;
						case 4:		// "ATNR0"
							lcd.clear();
							lcd.print("ZB NETWORK RESET");
							lcd.setCursor(0,1);
							lcd.print("NO <- OK? -> YES");
							while( lcd.readButtons() ) delay(100);	// 最後のキーが残っていた場合
							while( !(buttons = lcd.readButtons()) );	// ボタン待ち
							if( buttons == BUTTON_RIGHT ){
								lcd.clear();
								lcd.print("RESET = ");
								lcd.print( xbee_at("ATNR00") );	// ネットワークリセット
							} else {
								lcd.clear();
								lcd.print("Canceled");
							}
							delay(1000);
						default:
							break;
					}
					break;
				default:
					break;
			}
			while( lcd.readButtons() ) delay(100);	// 最後のキーが残っていた場合
			delay(500);
		}
	
		/* データ受信(待ち受けて受信する) */
		xbee_rx_call( &xbee_result );

		if( xbee_result.MODE != MODE_AUTO ){	// 何かを受信した
			lcd_clear |= 0x02;	// 次回のUI画面消去用のフラグのセット
			counter = 0;
			switch( xbee_result.MODE ){			// 受信したデータの内容(MODE値)に応じて
				case MODE_UART:		// シリアルデータを受信
					if( (lcd_clear&0x01) == 0x01 ){	// 前回の分が[enter]で終わっているときに画面消去
						lcd.clear();
						lcd_print_level( level_1, level_2);
						lcd_clear &= 0xFE;			// フラグのクリア
					}
					if( data >= 3 && xbee_result.DATA[0] == 0x1B ){	// ESCコードが先頭だったとき
						switch( xbee_result.DATA[1]  ){	
							case (byte)'L':
								if( xbee_result.DATA[2] >= 0x00 && 
									xbee_result.DATA[2] <= 0x0A )
										level_1 = xbee_result.DATA[2];
								if( xbee_result.DATA[2] >= (byte)'0' && 
									xbee_result.DATA[2] <= (byte)'0' )
										level_1 = xbee_result.DATA[2] - (byte)'0';
								if( xbee_result.DATA[2] >= 0x10 && 
									xbee_result.DATA[2] <= 0x1A )
										level_2 = xbee_result.DATA[2] - 16;
								if( xbee_result.DATA[2] >= (byte)'@' && 
									xbee_result.DATA[2] <= (byte)'I' )
										level_2 = xbee_result.DATA[2] - (byte)'@';
								lcd_print_level( level_1, level_2);
								break;
							case (byte)'B':
								// beep( xbee_result.DATA[2] );
								break;
							case (byte)'C':
								// chime( xbee_result.DATA[2] );
								break;
							case (byte)'A':
								alert_value = xbee_result.DATA[2];
								if( alert_value >= (byte)'0' && alert_value <= (byte)'9' ) alert_value -= (byte)'0';
								if( alert_value > 10 ) alert_value = 10 ;
								break;
							default:
								lcd.clear();
								lcd.setCursor(0,1);
								lcd.print("ESC(0x");
								if( xbee_result.DATA[1] < 16 ) lcd.print("0");
								lcd.print( xbee_result.DATA[1], HEX );
								lcd.print(")=0x");
								if( xbee_result.DATA[2] < 16 ) lcd.print("0");
								lcd.print( xbee_result.DATA[2], HEX );
								break;
						}
					}else{
						for(i=0;i<data;i++){
							if( xbee_result.DATA[i] >= (byte)' ' && xbee_result.DATA[i] <= 126){
								if( str_len < 16){
									strings[str_len] = (char)xbee_result.DATA[i];
									str_len++;
									strings[str_len] = '\0';
								}else{
									/* スクロール表示 */
									for( str_len = 1 ; str_len < 16 ; str_len++ )
										strings[str_len-1] = strings[str_len];
									strings[15] = (char)xbee_result.DATA[i];
									lcd.setCursor(0,1);
									lcd.print( strings );
									delay(500);
								}
							}
							if( xbee_result.DATA[i] == (byte)'\r' ||
								xbee_result.DATA[i] == (byte)'\n' ){
								lcd_clear |= 0x01;		// フラグのセット
								str_len = 0;
							}
						}
						lcd.setCursor(0,1);
						lcd.print( strings );
					}
					break;
				case MODE_GPIN:		// GPIOからのデータを受信
					lcd.clear();
					lcd.print( "recieved GPIN" );
					lcd.setCursor(0,1);
					for(i=0;i<8;i++)
						lcd.print( (xbee_result.DATA[0]>>(7-i))&0x01 , BIN );
					break;
				case MODE_RESP:		// リモートATコマンドの応答を受信
					lcd.clear();
					lcd.print( "recieved RESP" );
					lcd.setCursor(0,1);
					for(i=1;i<=3;i++){
						if( xbee_result.ADCIN[i] < 1024)
							lcd.print( xbee_result.ADCIN[i] , DEC );
						else lcd.print( "----" );
						lcd.print( " " );
					}
					break;
				case MODE_IDNT:		// 新しいデバイスを発見(表示のみ)
					lcd.clear();
					lcd.print( "recieved IDNT" );
					lcd.setCursor(0,1);
					for(i=0;i<8;i++){
						lcd.print( xbee_result.FROM[i]>>4 , HEX );
						lcd.print( xbee_result.FROM[i]%16 , HEX );
						dev_sens[i] = xbee_result.FROM[i];
					}
					break;
				default:
					lcd.clear();
					lcd.print( "MODE   = 0x" );
					lcd.print( xbee_result.MODE , HEX );
					lcd.setCursor(0,1);
					lcd.print( "STATUS = 0x" );
					lcd.print( xbee_result.STATUS , HEX);
					delay(500);
					break;
			}
		}
	}
}
